@@ -178,6 +178,10 @@ test_that("default usage formats S3 methods correctly", {
     call_to_usage("+.foo" <- function(x, b) {}),
     "\\method{+}{foo}(x, b)"
   )
+  expect_equal(
+    call_to_usage("%%.foo" <- function(x, b) {}),
+    "\\method{\\%\\%}{foo}(x, b)"
+  )
   expect_equal(
     call_to_usage("[<-.foo" <- function(x, value) {}),
     "\\method{[}{foo}(x) <- value"
@@ -193,6 +197,15 @@ test_that("S4 classes have no default usage", {
   )
 })
 
+test_that("default usage correct for S4 generics", {
+  expect_equal(
+    call_to_usage({
+      setGeneric("foo", function(x, y) {})
+    }),
+    "foo(x, y)"
+  )
+})
+
 test_that("default usage correct for S4 methods", {
   expect_equal(
     call_to_usage({
@@ -217,6 +230,14 @@ test_that("default usage correct for S4 methods", {
     }),
     "\\S4method{[}{Foo}(x, i, j, ...) <- value"
   )
+
+  expect_equal(
+    call_to_usage({
+      setGeneric("%&&%", function(x, y) standardGeneric("%&&%"))
+      setMethod("%&&%", signature("logical", "logical"), function(x, y) {})
+    }),
+    "\\S4method{\\%&&\\%}{logical,logical}(x, y)"
+  )
 })
 
 test_that("default usage correct for S4 methods with different args to generic", {
